
#ifndef ASE3HINCLUDED
#define ASE3HINCLUDED 1

/* Convert character cc to a visible one, 
	unprintable ones become a period. */ 
int showchar(int cc);

/* The "61 Hz" rate isn't quite 61 Hz, makes the measuring interval
	not quite 1 second. The real rate is ftick().  */ 
double ftick();

double secondscount(uint32_t taa);


/* Read and calculate the scale-factor, which is stored as
	a power of 10 (2s complement) byte in EEPROM at address 10. 
	This indicates how many decades we have to shift the accumuator
	value to get the original units.  Example, for a flowmeter with
	716 pulses per liter, the scale-factor is 100, as what is being
	accumulater is really centiliters, 7.16 pulses per centiliter that 
	is.
*/
int ascale(int fd, int ba, double *pascale);

int aunits(int fd, int ba, char *punit, size_t pumax);

/* Return accumulator value and time of accumulation */ 
int avalue(int fd, int ba, double *avalue, uint32_t *ptaa);

/* Convert any read tick-count to standard clock time. 
	avtime_x = 0 will be the time of last ticks-counter 
	reset.  */

time_t convertatime(int fd, int ba, 
	uint32_t avtime_x, 
	int *year, int *month, int *day, 
	int *hour, int *min, int *sec);

/* How long have we been running? */ 
double uptime(int fd, int ba);

/* Convert the tick-count to standard UNIX time. 
	avtime_x = 0 will be the time of last ticks-counter reset.  */

time_t tickunixtime(int fd, int ba, uint32_t avtime_x, double *puptime);


/* Measure the frequency. */
double fmeas(int fd, int ba);

/* Measure the analog input. */
int ameas(int fd, int ba);


/* Resets the accumulator. Sets it to 0 */
#define ARESET_0 0x10

/* Snapshot of the accumulator. 
	Updates its visible value into R_MSH...R_MCOUNTH
	Also makes a snapshot of the running clock into READSECONDS
	Aims the read-pointer to the accumulator to R_MSH
*/
#define ASNAPSHOT_0 0x11

/* Start the ADC. 
	Its results will appear in TX_ADCL,TX_ADCH
	Aims the read-pointer to TX_ADCL
*/

#define ADCSTART_0 0x14

/* Show the current time-count. 
	makes a snapshot of the running clock into READSECONDS
*/ 
#define TSNAPSHOT_0 0x15

/* Reset the tick-counter. */
#define TRESET_0 0x1c

/* Aim at some address in the EEPROM, to be read */
#define EEPROMADDRSET_1 0x18

/* Write: low byte is address, high byte is data */
#define EEPROMWRITE_2 0x19

/* Do Read, Copy eeprom data at the set address 
	to I2C_TX_EEPROMBYTE. The EEPROM read address 
	is autoincremented. */
#define EEPROMREAD_0 0x1A

#define SPACERD 0x40

/* Space: */

#define I2C_TX_ADCL   (SPACERD+0x00)
#define I2C_TX_ADCH   (SPACERD+0x01)

#define I2C_TX_EEPROMBYTE  (SPACERD+0x02)

/* Frequency count: how many pulses entered in timer1 
	during 61 ftick()s.  Real frequency = value * ftick() / 61.0; */ 

#define I2C_TX_TMR1L	(SPACERD+0x03) /* Frequency-counter LSB */ 
#define I2C_TX_TMR1H	(SPACERD+0x04) 
#define I2C_TX_TMR1C	(SPACERD+0x05) /* MSB */ 

#define I2C_TX_R_MSH     (SPACERD+0x06)
#define I2C_TX_R_MCOUNTL (SPACERD+0x07)
#define I2C_TX_R_MCOUNTM (SPACERD+0x08)
#define I2C_TX_R_MCOUNTH (SPACERD+0x09)

/* 1/61 or 1.0/ftick() second ticks count. */ 

/* When the last accumulator event happened */
#define I2C_TX_SECONDS0 (SPACERD+0x0A)
#define I2C_TX_SECONDS1 (SPACERD+0x0B)
#define I2C_TX_SECONDS2 (SPACERD+0x0C)
#define I2C_TX_SECONDS3 (SPACERD+0x0D)

/* Current tick at latest ASNAPSHOT or TSNAPSHOT action. */
#define I2C_TX_READSECONDS0 (SPACERD+0x0E)
#define I2C_TX_READSECONDS1 (SPACERD+0x0F)
#define I2C_TX_READSECONDS2 (SPACERD+0x10)
#define I2C_TX_READSECONDS3 (SPACERD+0x11)

#endif


