
/* eeafix.c -- Set a new i2c address in eeprom, eeprom address 0x0c 
	Range from 0x03-0x77, outside this range will be forced inside */ 

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <math.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <utilib.h>
#include "ase3.h"

#define I2CDEV "/dev/i2c-1"

int main(int argc, char **argv)
{
	char i2cdev[128];
	int fd, s, j, lo; 
	char *stop;
	int wz;

	int sa, sb;
	int a, b;

	sb = sa = 0;
	b = 0x77;
	a = 0;

	for(j = 1; j<argc; j++)
	{
		if(!strcmp("A", argv[j]))
		{
			if(j+1 < argc)
			{
				sa = 1;
				a = strtol(argv[j+1], &stop, 0); 
			}
		}

		if(!strcmp("B", argv[j]))
		{
			if(j+1 < argc)
			{
				sb = 1;
				b = strtol(argv[j+1], &stop, 0); 
			}
		}
	}
	
	if(sa) printf("Seen A=0x%04x=%d\n", a, a); 
	if(sb) printf("Seen B=0x%04x=%d\n", b, b); 
	else   printf("Default bus addr B=0x%04x=%d\n", b, b); 

	snprintf(i2cdev, 128, I2CDEV);

	fd = open(i2cdev, O_RDWR);
	if(fd == -1) 
	{
		perror(i2cdev); 
		return(-1); 
	}

	s = ioctl(fd, I2C_SLAVE, b); 
	if(s == -1) 
	{
		fprintf(stderr, "Bus address %d ", b); 
		perror("I2C_SLAVE");
		close(fd);
		return(-1);
	}

#if(0)
	if(sa)
	for(j = 0; j<256; j++)
	{
		wz = (a << 8) | j;
		delay(1500);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
	}
#endif

#if(1)

	/* Set the memory values for A */
	if(sa)
	{
		wz = (a << 8) | 12;
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
	}
#endif

	/* Read the byes for A, N, D, threshhold back. */

	delay(20);
	s = i2c_smbus_write_byte_data(fd, EEPROMADDRSET_1, 12);
	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	lo = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);

	printf("Read back A = 0x%04x = %d\n", lo, lo); 

	close(fd);

	return(0);
}


