
/* eedump.c -- Read and show the contents of the EEPROM 
	on the 12F1822 running the ase3.asm program. */ 

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <utilib.h>
#include "ase3.h"

#define I2CDEV "/dev/i2c-1"

int main(int argc, char **argv)
{
	char i2cdev[128];
	int fd, s, ea; 
	char *stop; 
	char txt[80];
	char t16[80];
	char theline[90];
	unsigned int vchar;
	int x, row, ba; 

	ea = 256;
	ba = 0x29;

	/* Optional end-address change */ 
	if(argc >= 2) ea = strtol(argv[1], &stop, 0); 

	snprintf(i2cdev, 128, I2CDEV);

	fd = open(i2cdev, O_RDWR);
	if(fd == -1) 
	{
		perror(i2cdev); 
		return(-1); 
	}

	s = ioctl(fd, I2C_SLAVE, ba); 
	if(s == -1) 
	{
		fprintf(stderr, "Bus address %d ", ba); 
		perror("I2C_SLAVE");
		close(fd);
		return(-1);
	}

	/* Commands: 
		i2c_smbus_write_byte_data(fd, EEPROMADDRSET_1, 0) to set the address
		i2c_smbus_write_byte(fd, EEPROMREAD_0) to copy the 
		byte read to the I2C_TX_EEPROMBYTE location.  
		Then Read this to get the actual value:
		= i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
		increments the address pointer.  */

	s = i2c_smbus_write_byte_data(fd, EEPROMADDRSET_1, 0);

	if(s == -1) 
	{
		fprintf(stderr, "Bus address %d ", ba); 
		perror("i2c_smbus_write_byte_data");
		close(fd);
		return(-1);
	}


	for(row = 0; row < 16; row++)
	{
		snprintf(theline, 90, "%02x: ", row * 16);

		for(x = 0; x<16; x++)
		{
			delay(20);
			i2c_smbus_write_byte(fd, EEPROMREAD_0);
			delay(20);
			vchar = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);

			snprintf(txt, 80, "%02x ", vchar); 
			strcat(theline, txt); 
			t16[x] = showchar(vchar);
		}
		t16[16] = 0; 
		strcat(theline, t16); 
		strcat(theline, "\n"); 
		fputs(theline, stdout); 

		if(row * 16 > ea) break;
	}

	close(fd);
	return(0);
}


