
/* eeset.c -- Put some values into the first 
	few cells in the EEPROM: 
	01   - A
	23   - N
	45   - D
	6789 - threshold (pic16F690)
	a    - decades
	b    - unit
	c    - i2c device address
*/

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <math.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <utilib.h>
#include "ase3.h"

#define I2CDEV "/dev/i2c-1"


int main(int argc, char **argv)
{
	char i2cdev[128];
	int fd, s, j; 
	char *stop;
	unsigned wz, a, n, c, d, u, aH, nH, dH, aL, nL, dL;
	int sa, sn, sd, sc, st, su;
	unsigned int hi, lo, tF, t0, t1, t2, ta, trd;
	double tpra, scale10;
	int8_t ci;
	int ba; 

	ba = 0x29; /* Default bus address... */

	sa = sn = sd = sc = st = su = 0;
	a = n = d = c = ta = u = 0;

	for(j = 1; j<argc; j++)
	{
		if(!strcmp("B", argv[j]))
		{
			if(j+1 < argc)
			{
				ba = strtol(argv[j+1], &stop, 0); 
			}
		}

		if(!strcmp("A", argv[j]))
		{
			if(j+1 < argc)
			{
				sa = 1;
				a = strtol(argv[j+1], &stop, 0); 
			}
		}

		if(!strcmp("N", argv[j]))
		{
			if(j+1 < argc)
			{
				sn = 1;
				n = strtol(argv[j+1], &stop, 0); 
			}
		}

		if(!strcmp("D", argv[j]))
		{
			if(j+1 < argc)
			{
				sd = 1;
				d = strtol(argv[j+1], &stop, 0); 
			}
		}
	
		if(!strcmp("C", argv[j]))
		{
			if(j+1 < argc)
			{
				sc = 1;
				c = strtol(argv[j+1], &stop, 0); 
			}
		}
	
		if(!strcmp("U", argv[j]))
		{
			if(j+1 < argc)
			{
				su = 1;
				u = argv[j+1][0]; 
			}
		}
	
		if(!strcmp("T", argv[j]))
		{
			if(j+1 < argc)
			{
				st = 1;
				ta = strtol(argv[j+1], &stop, 0); 
			}
		}
	}

	if(sa) printf("Seen A=0x%04x=%d\n", a, a); 
	if(sn) printf("Seen N=0x%04x=%d\n", n, n); 
	if(sd) printf("Seen D=0x%04x=%d\n", d, d); 
	if(sc) printf("Seen C=0x%04x=%d\n", c, c); 
	if(sc) printf("Seen U=0x%04x=%d=%c\n", u, u, u); 

	snprintf(i2cdev, 128, I2CDEV);

	fd = open(i2cdev, O_RDWR);
	if(fd == -1) 
	{
		perror(i2cdev); 
		return(-1); 
	}

	s = ioctl(fd, I2C_SLAVE, ba); 
	if(s == -1) 
	{
		fprintf(stderr, "Bus address %d ", ba); 
		perror("I2C_SLAVE");
		close(fd);
		return(-1);
	}

	/* Set the memory values for A */
	if(sa)
	{
		aH = a & 0xFF00;
		aL = (a & 0xFF) << 8;
		wz = aL | 0;
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);

		wz = aH | 1;
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
	}


	/* Set the memory values for N */
	if(sn)
	{
		nH = n & 0xFF00;
		nL = (n & 0xFF) << 8;
		wz = nL | 2;

		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
		wz = nH | 3;

		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
	}


	/* Set the memory values for D */
	if(sd)
	{
		dH = d & 0xFF00;
		dL = (d & 0xFF) << 8;
		wz = dL | 4; 
	
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
		wz = dH | 5;
		
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
	}

	if(sc)
	{
		wz = ((c & 0xFF) << 8) | 10;
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
	}

	if(su)
	{
		wz = ((u & 0xFF) << 8) | 11;
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
	}

	if(st)
	{
		tF = (ta & 0xFF) << 8;
		t0 = (ta & 0xFF00);
		t1 = (ta & 0xFF0000) >> 8;
		t2 = (ta & 0xFF000000) >> 16;

		printf("Write threshold %d.%d.%d.%d\n", t2, t1, t0, tF);

		wz = tF | 6; 
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);

		wz = t0 | 7; 
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);

		wz = t1 | 8; 
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);

		wz = t2 | 9; 
		delay(1000);
		s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, wz);
		printf("Write 0x%04x %d\n", wz, s);
	}

	/* Read the byes for A, N, D, threshhold back. */

	delay(20);
	s = i2c_smbus_write_byte_data(fd, EEPROMADDRSET_1, 0);
	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	lo = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	hi = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);

	a = lo + (hi << 8); 
	printf("Read back A = 0x%04x = %d\n", a, a); 

	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	lo = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	hi = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);

	n = lo + (hi << 8); 
	printf("Read back N = 0x%04x = %d\n", n, n); 

	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	lo = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	hi = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);

	d = lo + (hi << 8); 
	printf("Read back D = 0x%04x = %d\n", d, d); 

	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	tF = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	printf("Read back tF= 0x%04x = %d\n", tF, tF); 

	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	t0 = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	printf("Read back t0= 0x%04x = %d\n", t0, t0); 

	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	t1 = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	printf("Read back t1= 0x%04x = %d\n", t1, t1); 

	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	t2 = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	printf("Read back t2= 0x%04x = %d\n", t2, t2); 

	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	c = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	ci = c;
	scale10 = pow(10.0, (double) ci); 
	printf("Read back C = 0x%02x   = %d = %G\n", c, ci, scale10); 

	delay(20);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(20);
	u = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);
	printf("Read back U = 0x%02x   = \'%c\'\n", u, u); 

	trd = tF + (t0 << 8) + (t1 << 16) + (t2 << 24); 
	tpra = (double) trd / 256.0;
	printf("Read back threshold %u.%u.%u.%u = %u = %G\n", t2, t1, t0, tF, trd, tpra);
	printf("Practical threshold value = %G%c\n", tpra/scale10, u);

	close(fd);

	return(0);
}


