

/* pdinfo.c -- Look at the registers exposed in i2c in
	the 12F1822 running the ase3.asm program. */ 

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <utilib.h>
#include "ase3.h"

#define I2CDEV "/dev/i2c-1"


int main(int argc, char **argv)
{
	char i2cdev[128];
	int ba, fd, s; 
	char liste[20];
	char *stop; 
	int j, aa, bb;
	int ye, mo, da, ho, mi, se;
	double scale10;
	double upts;

	ba = 0x29;

	for(j = 1; j<argc; j++)
	{
		if(!strcmp("A", argv[j]))
		{
			if(j+1 < argc)
			{
				ba = strtol(argv[j+1], &stop, 0); 
			}
		}
	}

	snprintf(i2cdev, 128, I2CDEV);

	fd = open(i2cdev, O_RDWR);
	if(fd == -1) 
	{
		perror(i2cdev); 
		return(-1); 
	}

	ascale(fd, ba, &scale10); 

	s = ioctl(fd, I2C_SLAVE, ba); 
	if(s == -1) 
	{
		fprintf(stderr, "Bus address %d ", ba); 
		perror("I2C_SLAVE");
		close(fd);
		return(-1);
	}

	/* Issue some of the snapshot commands here first */ 
	s = i2c_smbus_write_byte(fd, ADCSTART_0); 
	if(s < 0)
	{
		fprintf(stderr, "smbus_write_byte ADCSTART_0"); 
		perror(i2cdev); 
	}
	delay(50);

	s = i2c_smbus_write_byte(fd, ASNAPSHOT_0); 
	if(s < 0)
	{
		fprintf(stderr, "smbus_write_byte ASNAPSHOT_0"); 
		perror(i2cdev); 
	}
	delay(50);

	s = i2c_smbus_write_byte_data(fd, EEPROMADDRSET_1, 0);
	delay(50);
	i2c_smbus_write_byte(fd, EEPROMREAD_0);
	delay(50);

	s = i2c_smbus_write_byte(fd, SPACERD); 
	if(s < 0)
	{
		fprintf(stderr, "smbus_write_byte SPACERD"); 
		perror(i2cdev); 
	}
	delay(50);

	s = read(fd, liste, 2); 
	aa = liste[0] + (liste[1] << 8);
	printf("ADC   : 0x%02x %02x       = %-8d = %G V\n", liste[ 1], liste[ 0], 
		aa, (double) aa * 2.048 / 1024.0);

	s = read(fd, liste, 1); 
	aa = liste[0];
	printf("EEPROM: 0x%02x          = %-8d = (%c)\n", aa, aa, showchar(aa));

	s = read(fd, liste, 3); 
	aa = liste[0] + (liste[1] << 8) + (liste[2] << 16) ;
	printf("FC    : 0x%02x %02x %02x    = %-8d = %G Hz\n",  liste[ 2], liste[ 1], liste[ 0], 
		aa, (double) aa * ftick() / 61.0);

	s = read(fd, liste, 4); 
	aa = liste[0] + (liste[1] << 8) + (liste[2] << 16) + (liste[3] << 24) ;
	printf("AC    : 0x%02x %02x %02x %02x = %-8d = %G\n", liste[3], liste[2], liste[1], liste[0], 
		aa, (double) aa / (256.0 * scale10) ); /* Subject to scale-factor and unit. */

	s = read(fd, liste, 4); 
	aa = liste[0] + (liste[1] << 8) + (liste[2] << 16) + (liste[3] << 24) ;
	s = read(fd, liste+10, 4); 
	bb = liste[10] + (liste[11] << 8) + (liste[12] << 16) + (liste[13] << 24) ;

	convertatime(fd, ba, aa, &ye, &mo, &da, &ho, &mi, &se);

	printf("TA    : 0x%02x %02x %02x %02x = %-8d = %u-%02u-%02u %02u:%02u:%02u\n", 
		liste[3], liste[2], liste[1], liste[0], aa,
		ye, mo, da, ho, mi, se);

	convertatime(fd, ba, bb, &ye, &mo, &da, &ho, &mi, &se);

	printf("TR    : 0x%02x %02x %02x %02x = %-8d = %u-%02u-%02u %02u:%02u:%02u\n", 
		liste[13], liste[12], liste[11], liste[10], bb,
		ye, mo, da, ho, mi, se);

	convertatime(fd, ba, 0, &ye, &mo, &da, &ho, &mi, &se);

	printf("Start :                          = %u-%02u-%02u %02u:%02u:%02u\n", 
		ye, mo, da, ho, mi, se);

	upts = uptime(fd, ba);
	printf("Uptime:                          = %G s\n", upts);

#if(0)
	for(j = 0; j<18; j++)
	{
		s = read(fd, liste+j, 1); 

		if(s < 0)
		{
			fprintf(stderr, "read 18:"); 
			perror(i2cdev); 
		}
		delay(10);
	}

	printf("EEPROM=0x%02x (%c)\n",       liste[ 3], showchar(liste[3]));
	printf("FC=0x%02x %02x %02x\n",      liste[ 6], liste[ 5], liste[ 4]);
	printf("AC=0x%02x %02x %02x %02x\n", liste[ 9], liste[ 8], liste[ 7], liste[ 6]);
	printf("TA=0x%02x %02x %02x %02x\n", liste[13], liste[12], liste[11], liste[10]);
	printf("TR=0x%02x %02x %02x %02x\n", liste[17], liste[16], liste[15], liste[14]);
#endif

#if(0)
	printf("About to write spacerd byte\n");
	s = i2c_smbus_write_byte(fd, SPACERD); 

	if(s < 0)
	{
		fprintf(stderr, "smbus_write_byte "); 
		perror(i2cdev); 
	}

	printf("Done write spacerd byte = %d\n", s);

	delay(50);
#endif

#if(0)
	printf("About to read %d bytes\n", 16);
	for(j = 0; j<16; j++)
	{
		s = read(fd, liste+j, 1); 

		if(s < 0)
		{
			fprintf(stderr, "Read "); 
			perror(i2cdev); 
		}
		delay(50);
	}
#endif

#if(0)
	for(j = 0; j<16; j++)
	{
		printf("Read byte %d = %d\n", j, liste[j]); 
	}
#endif

	close(fd);
	return(0);
}


