
/* servase3 -- Serv the actions of the ASE3 chip,
	as well as other status info in the Raspberry Pi.

00 01       : ADC     : 0x00 35       = 53       = 0.106 V
02          : EEPROM  : 0xc1          = 193      = (.)
03 04 05    : FC      : 0x00 9a b3    = 39603    = 39625.8 Hz
06 07 08 09 : AC      : 0x00 00 00 47 = 71       = 0.00277344
0a 0b 0c 0d : Aevent  : 0x00 92 e8 0b = 9627659  = 2014-11-25 19:18:44
0e 0f 10 11 : Snapshot  : 0x00 94 18 98 = 9705624  = 2014-11-25 19:40:02
Start       :                          = 2014-11-23 23:29:45
Uptime      :                          = 159017 s

*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <linux/i2c-dev.h>
#include <utilib.h>
#include <tcpserver.h>
#include "ase3.h"

#define I2CDEV "/dev/i2c-1"

struct ase3struct
{
	char i2cdev[128];
	int ba;
	int verbose;
};

int servase3func(int csd, struct ase3struct *A3);


/* Listen on some port defaulting to 9813 */
int main(int argc, char **argv)
{
    unsigned short portno; 
    int j;
    char *stop; 

	struct ase3struct thease3, *A3;
	A3 = &(thease3);

	snprintf(A3->i2cdev, 128, "%s", I2CDEV);

    /* Default port number. */ 
    portno = 9813;
    A3->verbose = 0; 
	A3->ba = 41;

    /* Work through the args. 
        -p portno
		-b busaddr
		-d i2cdevice
        -v 
    */ 
    for(j = 1; j<argc; j++)
    {
        if(!strcmp("-v", argv[j])) A3->verbose = 1; 
        else
        if(!strcmp("-p", argv[j])) 
        {
            if(j+1 < argc)
                portno = (unsigned short) strtol(argv[j+1], &stop, 10); 
        }
        else
        if(!strcmp("-b", argv[j])) 
        {
            if(j+1 < argc)
                A3->ba = (unsigned short) strtol(argv[j+1], &stop, 10); 
        }
        else
        if(!strcmp("-d", argv[j])) 
        {
            if(j+1 < argc)
				snprintf(A3->i2cdev, 128, "%s", argv[j+1]);
		}
    }

    if(portno == 0)
    {
        fprintf(stderr, "Usage: %s -p port -b busaddr -v -d i2cdevice\n", argv[0]); 
        return(1);
    }

	if(A3->verbose)
	{
		fprintf(stderr, "Device=%s ba=%d Port=%d\n", 
			A3->i2cdev, A3->ba, portno); 
	}

    TCPserverSingle(portno, A3->verbose, servase3func, A3);

    return(0);
}

/* The function returns 0 to continue 
	running and 1 to terminate. */ 

int servase3func(int csd, struct ase3struct *A3)
{
	char rbuff[512];
	char txbuff[512];
	char ebuff[128];
	char cmd[128];
	char *stop; 
	int nrd; 

    memset(rbuff, 0, 512); 
    nrd = recv(csd, rbuff, 512, 0); 

	if(nrd == 0) return(0);
	if(nrd <= 0) return(1);

	stop = rbuff;

	for(;;)
	{
		strword(stop, &stop, cmd, 128); 
		if(notext(cmd)) break;

		if(!strcmp("?", cmd))
		{
			snprintf(txbuff, 512, "%s", 
				"Commands: ?/quit/radc/fval/aval/atime/afull/"
				"areset/eeread a/eewrite a d/start/uptime\n"); 

			send(csd, txbuff, strlen(txbuff), 0); 
		}

		if(!strcmp("quit", cmd)) return(1); 

		// -rwxr-xr-x 1 pi pi 11910 Nov 25 19:31 radc
		if(!strcmp("radc", cmd)) 
		{
			int fd; 
			int aa;

			fd = open(A3->i2cdev, O_RDWR);

			if(fd >= 0)
			{
				aa = ameas(fd, A3->ba);
				close(fd);
				snprintf(txbuff, 512, "Read A = %d\n", aa); 
			}
			else
			{
				aa = -1;
				strerror_r(errno, ebuff, 128);
				snprintf(txbuff, 512, "Read A = %d (%s ba=%d: %s)\n", 
					aa, A3-> i2cdev, A3-> ba, ebuff); 
			}

			send(csd, txbuff, strlen(txbuff), 0); 
		}

		// -rwxr-xr-x 1 pi pi 12032 Nov 25 19:31 fval
		if(!strcmp("fval", cmd)) 
		{
			int fd; 
			double fa;
			char fbuff[20];

			fd = open(A3->i2cdev, O_RDWR);

			if(fd >= 0)
			{
				fa = fmeas(fd, A3->ba); 
				close(fd);

				engrprint(fbuff, 20, fa, 3, "Hz"); 
				snprintf(txbuff, 512, "Read F = %s\n", fbuff); 
			}
			else
			{
				snprintf(fbuff, 20, "NAN Hz");
				strerror_r(errno, ebuff, 128);
				snprintf(txbuff, 512, "Read F = %s (%s ba=%d: %s)\n", 
					fbuff, A3->i2cdev, A3->ba, ebuff); 
			}

			send(csd, txbuff, strlen(txbuff), 0); 
		}

		// -rwxr-xr-x 1 pi pi 12622 Nov 25 19:31 aval
		if(!strcmp("aval", cmd) 
		|| !strcmp("atime", cmd) 
		|| !strcmp("afull", cmd)) 
		{
			int fd; 
			char autxt[10];
			double aval;
			uint32_t atx; 
			int ye, mo, da, ho, mi, se;
			char abuff[30];

			fd = open(A3->i2cdev, O_RDWR);

			if(fd >= 0)
			{
				/* Return accumulator value and time of accumulation */ 
				avalue(fd, A3->ba, &aval, &atx); 

				if(!strcmp("aval", cmd))
				{
					aunits(fd, A3->ba, autxt, 10); 
					engrprint(abuff, 30, aval, 3, autxt); 
					snprintf(txbuff, 512, "Read A = %s\n", abuff);
				}

				if(!strcmp("atime", cmd)) 
				{
					convertatime(fd, A3->ba, atx, &ye, &mo, &da, &ho, &mi, &se);
					snprintf(txbuff, 512, 
						"AT = %u-%02u-%02u %02u:%02u:%02u\n", 
						ye, mo, da, ho, mi, se);
				}

				if(!strcmp("afull", cmd))
				{
					aunits(fd, A3->ba, autxt, 10); 
					engrprint(abuff, 30, aval, 3, autxt); 
					convertatime(fd, A3->ba, atx, &ye, &mo, &da, &ho, &mi, &se);

					snprintf(txbuff, 512, 
						"Read A = %s at %u-%02u-%02u %02u:%02u:%02u\n", 
						abuff, ye, mo, da, ho, mi, se);
				}
				close(fd);
			}
			else
			{
				snprintf(abuff, 30, "No A");
				strerror_r(errno, ebuff, 128);
				snprintf(txbuff, 512, "Read A = %s (%s ba=%d: %s)\n", 
					abuff, A3->i2cdev, A3->ba, ebuff); 
			}

			send(csd, txbuff, strlen(txbuff), 0); 
		}

		if(!strcmp("areset", cmd))
		{
			int fd, s, rx; 

			fd = open(A3->i2cdev, O_RDWR);

			if(fd >= 0)
			{
				s = ioctl(fd, I2C_SLAVE, A3->ba); 
				if(s >= 0)
				{
					rx = i2c_smbus_write_byte(fd, ARESET_0); 
					if(rx >= 0)
					{
						snprintf(txbuff, 512, "Reset A \n" );  
					}
					else
					{
						strerror_r(errno, ebuff, 128);
						snprintf(txbuff, 512, "Reset A (%s ba=%d: %s)\n", 
							A3->i2cdev, A3->ba, ebuff); 
					}
				}
				else
				{
					strerror_r(errno, ebuff, 128);
					snprintf(txbuff, 512, "Reset A (%s ba=%d: %s)\n", 
						A3->i2cdev, A3->ba, ebuff); 
				}
					
				close(fd);
			}
			else
			{
				strerror_r(errno, ebuff, 128);
				snprintf(txbuff, 512, "Reset A (%s ba=%d: %s)\n", 
					A3->i2cdev, A3->ba, ebuff); 
			}
			send(csd, txbuff, strlen(txbuff), 0); 
		} 

		/* EEprom: set address and read or write:
			eeread addr 
		*/
		if(!strcmp("eeread", cmd))
		{
			int fd, s, addr, drr; 

			addr = strtol(stop, &stop, 0);

			fd = open(A3->i2cdev, O_RDWR);

			if(fd >= 0)
			{
				s = ioctl(fd, I2C_SLAVE, A3->ba); 
				if(s >= 0)
				{
					delay(20);
					i2c_smbus_write_byte_data(fd, EEPROMADDRSET_1, addr); 
					delay(20);
					i2c_smbus_write_byte(fd, EEPROMREAD_0); 
					delay(20);
					drr = i2c_smbus_read_byte_data(fd, I2C_TX_EEPROMBYTE);

					snprintf(txbuff, 512, "Read E(0x%02x) = 0x%02x\n",
						addr & 0x0FF, drr & 0x0FF );  
				}
				else
				{
					strerror_r(errno, ebuff, 128);
					snprintf(txbuff, 512, "Read E(0x%02x) (%s ba=%d: %s)\n", 
						addr & 0x0FF, A3->i2cdev, A3->ba, ebuff); 
				}
					
				close(fd);
			}
			else
			{
				strerror_r(errno, ebuff, 128);
				snprintf(txbuff, 512, "Read E(0x%02x) (%s ba=%d: %s)\n", 
					addr & 0x0FF, A3->i2cdev, A3->ba, ebuff); 
			}
			send(csd, txbuff, strlen(txbuff), 0); 
		}

		/* EEprom: set address and read or write:
			eewrite addr value
		*/
		if(!strcmp("eewrite", cmd))
		{
			int fd, s, addr, dw, za; 

			addr = strtol(stop, &stop, 0);
			dw = strtol(stop, &stop, 0);
			za = (addr & 0xFF) | ( (dw &0x0FF) << 8);

			fd = open(A3->i2cdev, O_RDWR);

			if(fd >= 0)
			{
				s = ioctl(fd, I2C_SLAVE, A3->ba); 
				if(s >= 0)
				{
					delay(1000);
					s = i2c_smbus_write_word_data(fd, EEPROMWRITE_2, za);

					snprintf(txbuff, 512, "Write E(0x%02x) to 0x%02x (0x%x) = %d\n",
						addr & 0x0FF, dw & 0x0FF, za, s );  
				}
				else
				{
					strerror_r(errno, ebuff, 128);
					snprintf(txbuff, 512, 
						"Write E(0x%02x) to 0x%02x (%s ba=%d: %s)\n", 
						addr & 0x0FF, dw & 0x0FF, 
						A3->i2cdev, A3->ba, ebuff); 
				}
					
				close(fd);
			}
			else
			{
				strerror_r(errno, ebuff, 128);
				snprintf(txbuff, 512, 
					"Write E(0x%02x) to 0x%02x (%s ba=%d: %s)\n", 
					addr & 0x0FF, dw & 0x0FF, 
					A3->i2cdev, A3->ba, ebuff); 
			}
			send(csd, txbuff, strlen(txbuff), 0); 
		}

		/* start */
		if(!strcmp("start", cmd))
		{
			int fd; 
			int ye, mo, da, ho, mi, se;

			fd = open(A3->i2cdev, O_RDWR);

			if(fd >= 0)
			{
				convertatime(fd, A3->ba, 0, &ye, &mo, &da, &ho, &mi, &se);
				close(fd);

				snprintf(txbuff, 512, 
					"Start = %u-%02u-%02u %02u:%02u:%02u\n", 
					ye, mo, da, ho, mi, se);
			}
			else
			{
				strerror_r(errno, ebuff, 128);
				snprintf(txbuff, 512, "Read Start = %d (%s ba=%d: %s)\n", 
					-1, A3-> i2cdev, A3-> ba, ebuff); 
			}

			send(csd, txbuff, strlen(txbuff), 0); 
		}

		/* uptime */ 
		if(!strcmp("uptime", cmd))
		{
			int fd; 
			int aa;
			// char abuff[30];

			fd = open(A3->i2cdev, O_RDWR);

			if(fd >= 0)
			{
				aa = uptime(fd, A3->ba);
					// engrprint(abuff, 30, aval, 3, "s"); 
					// snprintf(txbuff, 512, "Read U = %s\n", abuff);
				close(fd);
				snprintf(txbuff, 512, "Read U = %d s\n", aa); 
			}
			else
			{
				aa = -1;
				strerror_r(errno, ebuff, 128);
				snprintf(txbuff, 512, "Read U = %d (%s ba=%d: %s)\n", 
					aa, A3->i2cdev, A3->ba, ebuff); 
			}

			send(csd, txbuff, strlen(txbuff), 0); 
		}

	}/* loop over all command-strings */ 

	return(0); 
}

