
#define _BSD_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include "timdat.h" 

/* Set the time-of-day from arguments, via a system issue. Slow and 
	silly-looking, but it works. */ 
void setdt_command(int year, int mo, int da, int ho, int mi) 
{
	char datecmd[100];

	snprintf(datecmd, 100, 
		"date %02u%02u%02u%02u%04u > /dev/null", 
		mo, da, ho, mi, year); 

	system(datecmd);
}

/* Take a struct tm and return its constituent parts. 
	Recalculates year and month, so they have the conventional
	values (4 digits, and 1-12) */ 
	
void choptm(struct tm *ptm, 
	int *year, int *mo, int *da, int *ho, int *mi, int *se)
{
	*year = ptm->tm_year + 1900;
	*mo = ptm->tm_mon + 1;
	*da = ptm->tm_mday;

	*ho = ptm->tm_hour;
	*mi= ptm->tm_min;
	*se = ptm->tm_sec;
}

/* Get today's date and time into individual variables */ 
void getdt(int *year, int *mo, int *da, int *ho, int *mi, int *se)
{
	struct tm *pnow;
	time_t timer;
	
	time(&timer);
	pnow = localtime(&timer);
	choptm(pnow, year, mo, da, ho, mi, se); 
} 


/* Assmeble the year etc. into a struct tm, but does not set wday or yday. */ 
void assytm(struct tm *ta, int year, int mo, int da, int ho, int mi, int se)
{
	ta->tm_year = year - 1900;
	ta->tm_mon = mo-1;
	ta->tm_mday = da;
	ta->tm_hour = ho;
	ta->tm_min = mi;
	ta->tm_sec = se;
	ta->tm_wday = 0; /* Ignored on entry, set on return */ 
	ta->tm_yday = 0;    /* Ignored on entry, set on return */ 
	ta->tm_isdst = 0;
}

/* Convert time in ISO format (YYYY-MM-DDXHH:MM:SS) to a 
	time_t and a struct tm (the struct tm does not have weekday
	or yearday set right away) */ 
time_t isotime(char *isorep, struct tm *ptm)
{
	char *stop;
	struct tm ta; 
	time_t trr;
	int val;

	val = strtol(isorep, &stop, 10); 
	ta.tm_year = val - 1900;
	val = strtol(stop+1, &stop, 10); 
	ta.tm_mon = val-1;

	ta.tm_mday = strtol(stop+1, &stop, 10); 
	ta.tm_hour = strtol(stop+1, &stop, 10); 
	ta.tm_min = strtol(stop+1, &stop, 10); 
	ta.tm_sec = strtol(stop+1, &stop, 10); 

	ta.tm_wday = 0; /* Ignored on entry, set on return */ 
	ta.tm_yday = 0; /* Ignored on entry, set on return */ 
	ta.tm_isdst = 0;

	if(ptm) *ptm = ta; 
	trr = mktime(&ta);

	return(trr);
}


/* Set day and time via the settimeofday() system function */ 
int setdt(int year, int mo, int da, int ho, int mi, int se) 
{
	int ts;
	struct timeval thetv;
	struct timezone thetz; 
	struct tm ta; 

	assytm(&ta, year, mo, da, ho, mi, se); 

	thetv.tv_sec = mktime(&ta);
	thetv.tv_usec = 0; 

	thetz.tz_minuteswest = 0;
	thetz.tz_dsttime = 0;

	ts = settimeofday(&thetv, &thetz);

	return ts;
}






